//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see http://www.gnu.org/licenses/.
//
// @author Brian Ricks
//

#ifndef RADIOPACKETGENERATOR_H_
#define RADIOPACKETGENERATOR_H_

#include "INotifiable.h"
#include "NotificationBoard.h"
#include "RadioState.h"

// Generates packets for direct transmission by a radio.  No MAC is assumed.
// Useful for simulation of jammers.
class RadioPacketGenerator : public cSimpleModule, public INotifiable {
public:
    RadioPacketGenerator();
    virtual ~RadioPacketGenerator();

protected:
    NotificationBoard *nb;  // Used for notification of radio state changes

    virtual void initialize(int stage);
    virtual void handleMessage(cMessage *msg);
    virtual void finish();

    // contract from INotifiable
    virtual void receiveChangeNotification(int category, const cObject *details);

private:
    RadioState::State _radioState;  // stores the current radio state
    int _radioModuleID;

    cMessage *scheduleMsg;

    long generatedPktLength;  // in bytes
    simtime_t sendDelay; // delay between packet sending, in seconds

    bool isScheduled; // if we have a packet waiting to be sent, this will be true

    void sendPacket();
};

#endif /* RADIOPACKETGENERATOR_H_ */
